import {
  Metadata, dayjs as moment, I18n, IEventParam, Loader, IDictionary, Router, DatasourceFactory,
} from '@zeedhi/core';
import {
  IterableComponentRender, Form, Text, Switch, Select, DialogService, LoadingService, Button, Card, Tabs, Tab, Increment,
} from '@zeedhi/common';
import { BottomSheet } from '@zeedhi/zd-bottom-sheet';
// import { getStoredData } from '@zeedhi/zeedhi-miragejs';
import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';

export class SolicitacaoComprasController {
  private utilities = Loader.getInstance('Utilities');

  private appController = Loader.getInstance('AppController');

  public showAddProduto = false;

  public showSCDetail = false;

  private periodo = {
    DTINICIAL: '',
    DTFINAL: '',
  };

  private filiParams!: IDictionary;

  get isVisibleDetailList() {
    return !this.showAddProduto && !this.showSCDetail;
  }

  public get setFilterSC() {
    if (localStorage.getItem('hasImportedData') === 'true') {
      return {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        DTPROGENTR: this.utilities.getAllDatesInRange(this.periodo.DTINICIAL, this.periodo.DTFINAL),
      };
    }
    return {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      DTINICIAL: this.periodo.DTINICIAL || '',
      DTFINAL: this.periodo.DTFINAL || '',
    };
  }

  public setValueQtde({ component }: IEventParam<Text>) {
    const value = component.name.split('-').reverse()[0] || '';
    const mask = {
      decimalPlaces: 5,
    };
    const maskNumber = this.utilities.getMaskNumber(value, mask);
    component.text = `${I18n.translate('Qtde')}: ${maskNumber}`;
  }

  public setValueQtdeAjustada({ component }: IEventParam<Text>) {
    const value = component.name.split('-').reverse()[0] || '';
    if (value === 'null') component.text = `${I18n.translate('Qtde Ajustada')}: `;
    else {
      const mask = {
        decimalPlaces: 5,
      };
      const maskNumber = this.utilities.getMaskNumber(value, mask);
      component.text = `${I18n.translate('Qtde Ajustada')}: ${maskNumber}`;
    }
  }

  public openAddProdutoSC() {
    const form = Metadata.getInstance<Form>('addProdutoSCForm');
    form.clearForm();
    const component = Metadata.getInstance<Switch>('PRODUTOEXTRA');
    component.value = false;
    this.disableJustExtra({ component });
    this.showAddProduto = true;
    // this.setNotPossibleDates();
  }

  public clickBackAddProduto() {
    this.showAddProduto = false;
    const cardSC = Metadata.getInstance<IterableComponentRender>('iterableCardsSCompras');
    cardSC.reload();
  }

  public async clickCardSC({ component }: IEventParam<Card>) {
    LoadingService.show(' ');
    this.showSCDetail = true;
    const tabsSC = Metadata.getInstance<Tabs>('tabsSC');
    const scRender = Metadata.getInstance<IterableComponentRender>('iterableCardsSCompras');
    const ds = DatasourceFactory.factory({
      uniqueKey: 'CDFILIAL',
      type: 'rest',
      route: '/parametros',
      filter: {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      },
    });
    let param: IDictionary;
    await ds.get().then((result: IDictionary) => {
      // param = result[0];
      param = result.slice(0, 1).shift();
    });
    if (tabsSC.tabs.length > 0) {
      await Promise.all(tabsSC.tabs.map(async (tab: IDictionary) => {
        await tab.children.map(async (child: IDictionary) => {
          await tab.removeChild(child.name);
        });
      })).then(() => {
        tabsSC.tabs = [];
        tabsSC.tabs = scRender.datasource.data.map((row) => this.newTabs(row, param));
      });
    } else tabsSC.tabs = scRender.datasource.data.map((row) => this.newTabs(row, param));
    const index = component.name.split('-')[1];
    tabsSC.activeTab = await tabsSC.tabs.findIndex((tab) => tab.children[0].name === `tagTab-${index}`);
    LoadingService.hide();
  }

  public newTabs(row: IDictionary, param: IDictionary) {
    const isVisibleAux = (moment(row.DTPROGENTR, 'DD/MM/YYYY') <= moment() || (row.IDSOLIAPROV === 'S'
      && param.ALTERAR_PRODUTO_APROVADO === 'N'));
    const mask = {
      decimalPlaces: 5,
    };
    const qtAjustada = isVisibleAux
      ? {
        allowDuplicate: true,
        name: 'QTAJUSTADA',
        component: 'ZdText',
        isVisible: isVisibleAux,
        text: `${I18n.translate('Qtde Ajustada')}: ${row.QTAJUSTADA
          ? this.utilities.getMaskNumber(row.QTAJUSTADA || '', mask) : ''}`,
        grid: {
          cols: 12,
        },
      }
      : {
        allowDuplicate: true,
        name: 'QTAJUSTADA',
        component: 'ZdIncrement',
        label: `${I18n.translate('Qtde Ajustada')}: `,
        clearable: false,
        step: 1,
        isVisible: !isVisibleAux,
        // text: `${I18n.translate('Quantidade Ajustada')}: ${this.utilities.getMaskNumber(row.QTAJUSTADA
        //   || '', mask)}`,
        events: {
          change: '{{SolicitacaoComprasController.changeQuantAjustada}}',
        },
        validations: {
          min: {
            limit: 0.1,
          },
        },
        value: row.QTAJUSTADA,
        grid: {
          cols: 12,
        },
      };
    const dsjustoper = isVisibleAux
      ? {
        allowDuplicate: true,
        name: 'DSJUSTOPER',
        component: 'ZdText',
        isVisible: isVisibleAux,
        text: `${I18n.translate('Justificativa Quantidade')}: ${row.DSJUSTOPER ? row.DSJUSTOPER : ''}`,
        grid: {
          cols: 12,
        },
      }
      : {
        allowDuplicate: true,
        name: 'DSJUSTOPER',
        component: 'ZdSelect',
        label: `${I18n.translate('Justificativa Quantidade')}: `,
        isVisible: !isVisibleAux,
        // text: `${I18n.translate('Justificativa Quantidade')}: ${row.DSJUSTOPER ? row.DSJUSTOPER : ''}`,
        returnObject: true,
        datasource: {
          uniqueKey: 'CDJUSTSOL',
          route: '/justificativas',
          type: 'rest',
          limit: '10',
          lazyLoad: true,
        },
        dataValue: 'DSJUSTOPER',
        dataText: ['DSJUSTOPER'],
        formatterDataText: ['DSJUSTOPER'],
        events: {
          change: '{{SolicitacaoComprasController.changeJustQuant}}',
        },
        value: row.DSJUSTOPER,
        validations: {},
        grid: {
          cols: 12,
        },
      };
    return new Tab({
      name: `${row.KEY}`,
      tabTitle: `${this.utilities.getValueWithMaskProduto(row.CDARVPROD)} - ${row.NMPRODUTO}`,
      children: [
        {
          allowDuplicate: true,
          name: `tagTab-${row.KEY}`,
          component: 'ZdTag',
          tag: 'div',
          children: [
            {
              name: `formDetailSC-${row.CDPRODUTO}`,
              component: 'ZdForm',
              children: [
                {
                  allowDuplicate: true,
                  name: 'CDPRODUTO',
                  component: 'ZdText',
                  text: `${I18n.translate('Produto')}: ${this.utilities.getValueWithMaskProduto(row.CDARVPROD)} ${' | '}
                  ${row.NMPRODUTO}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'SGUNIDADE',
                  component: 'ZdText',
                  text: `${I18n.translate('Unidade')}: ${row.SGUNIDADE}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'DTPROGENTR',
                  component: 'ZdText',
                  text: `${I18n.translate('Data de Entrega')}: ${this.utilities.formatDate(row.DTPROGENTR)}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'IDSOLIAPROV',
                  component: 'ZdText',
                  text: `${I18n.translate('Aprovado')}: ${row.IDSOLIAPROV === 'N'
                    ? I18n.translate('Não') : I18n.translate('Sim')}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'QTSOLIPROD',
                  component: 'ZdText',
                  text: `${I18n.translate('Quantidade')}: ${this.utilities.getMaskNumber(row.QTSOLIPROD || '', mask)}`,
                  grid: {
                    cols: 12,
                  },
                },
                qtAjustada,
                dsjustoper,
                {
                  allowDuplicate: true,
                  name: 'NRSOLICMP',
                  component: 'ZdText',
                  text: `${I18n.translate('Solicitação')}: ${row.NRSOLICMP ? row.NRSOLICMP : ''}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'IDITMSOLEXTRA',
                  component: 'ZdText',
                  text: `${I18n.translate('Produto Extra')}: ${row.IDITMSOLEXTRA === 'N'
                    ? I18n.translate('Não') : I18n.translate('Sim')}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'DSJUSTSOLEXT',
                  component: 'ZdText',
                  text: `${I18n.translate('Justificativa Extra')}: ${row.DSJUSTSOLEXT ? row.DSJUSTSOLEXT : ''}`,
                  grid: {
                    cols: 12,
                  },
                },
              ],
            },
            {
              name: `salvarEditProdSCBtn_${row.KEY}`,
              component: 'ZdButton',
              label: 'Salvar',
              block: true,
              fixed: true,
              bottom: true,
              disabled: true,
              ligth: false,
              dark: true,
              events: {
                click: '{{SolicitacaoComprasController.salvarEditProdSC}}',
              },
              cssStyle: {
                right: '0px',
              },
            },
          ],
        },
      ],
    });
  }

  public setFormattedDate({ component }: IEventParam<Text>) {
    const value = component.name.split('-')[1] || '';
    component.text = `${I18n.translate('Data de Entrega')}: ${this.utilities.formatDate(value)}`;
    // text: Data de Entrega : <<row.DTPROGENTAF>>
  }

  public disableJustExtra({ component }: IEventParam<Switch>) {
    const justExtra = Metadata.getInstance<Select>('DSJUSTSOLEXT');
    if (component.value === true) {
      justExtra.isVisible = true;
      justExtra.addValidation('required', { message: ' ' });
    } else {
      justExtra.isVisible = false;
      justExtra.removeValidation('required');
    }
  }

  public allowedDates(date: string) {
    const today = new Date();
    const inputDate = new Date(date);

    return inputDate >= today;
  }

  public salvaProdutoSC() {
    const form = Metadata.getInstance<Form>('addProdutoSCForm');
    if (form.validate()) {
      const valueForm = form.value;
      const today = new Date().toJSON().slice(0, 10);
      const todayHour = new Date().toJSON().slice(0, 19).replace('T', ' ');

      if (valueForm.PRODUTOEXTRA === undefined || valueForm.PRODUTOEXTRA === null
        || valueForm.PRODUTOEXTRA === false) valueForm.IDITMSOLEXTRA = 'N';
      else valueForm.IDITMSOLEXTRA = 'S';

      const dataset = [
        {
          CDFILLAN: localStorage.getItem('CDFILLANCA') || '',
          CDFILSOLI: localStorage.getItem('CDFILIALLOGADA') || '',
          CDPRODUTO: valueForm.CDPRODUTO.CDPRODUTO,
          CDARVPROD: valueForm.CDPRODUTO.CDARVPROD,
          DSJUSTOPER: valueForm.DSJUSTOPER.DSJUSTOPER,
          DTPROGENTR: moment(valueForm.DTPROGENTR, 'YYYY-MM-DD').format('DD/MM/YYYY'),
          DTUTILIZA: moment(valueForm.DTPROGENTR, 'YYYY-MM-DD').format('DD/MM/YYYY'),
          NMPRODUTO: valueForm.CDPRODUTO.NMPRODUTO,
          CDFILLANCA: localStorage.getItem('CDFILLANCA') || '',
          NRSOLICMP: I18n.translate('scm::TO_BE_CREATED'),
          IDITMSOLEXTRA: valueForm.IDITMSOLEXTRA,
          DSJUSTSOLEXT: valueForm.DSJUSTSOLEXT?.DSJUSTOPER || null,
          QTAJUSTADA: null,
          QTSOLIPROD: valueForm.QTSOLIPROD,
          ROWNUM: 'L1',
          SGUNIDADE: valueForm.CDPRODUTO.SGUNIDADE,
          IDSOLIAPROV: 'N',
          CDFILIOPELOG: null,
          // campos da solicitação
          DTSOLCMP: moment(today, 'YYYY-MM-DD').format('DD/MM/YYYY'),
          IDTIPLAN: 'M',
          IDSOLEXTRA: valueForm.IDITMSOLEXTRA,
          CDFILCENT: localStorage.getItem('CDFILCENTCMP') || '',
          CDOPERLANC: localStorage.getItem('CDOPERADOR'),
          DTHRLANCSOL: moment(todayHour, 'YYYY-MM-DD HH:mm:ss').format('DD/MM/YYYY HH:mm:ss'),
          QTITEMSOLIC: 'Definir na exportação',
          changed: true,
          CDFILICONCATNRSOL: '',
          LOG: {
            CDFILSOLMOV: localStorage.getItem('CDFILIALLOGADA') || '',
            CDFILLANMOV: localStorage.getItem('CDFILLANCA') || '',
            NRSOLIMOV: I18n.translate('scm::TO_BE_CREATED'),
            CDPRODUTO: valueForm.CDPRODUTO.CDPRODUTO,
            DTPROGENTR: moment(valueForm.DTPROGENTR, 'YYYY-MM-DD').format('DD/MM/YYYY'),
            DTOPERSOL: moment(today, 'YYYY-MM-DD').format('DD/MM/YYYY'),
            DSJUSTOPER: valueForm.DSJUSTOPER.DSJUSTOPER || null,
            DSJUSTSOLEXT: valueForm.DSJUSTSOLEXT?.DSJUSTOPER || null,
            QTANTOPER: valueForm.QTSOLIPROD,
            QTDEPOPER: null,
            IDOPERAMOVS: 'I',
            IDITMSOLEXTRA: valueForm.IDITMSOLEXTRA,
            changed: true,
          },
        },
      ];

      const scsRender = Metadata.getInstance<IterableComponentRender>('iterableCardsSCompras');
      const dsConfig = scsRender.datasource.clone();
      dsConfig.loadAll = true;
      dsConfig.filter = {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        DTPROGENTR: moment(valueForm.DTPROGENTR, 'YYYY-MM-DD').format('DD/MM/YYYY') || '',
      };
      const clonedDs = DatasourceFactory.factory(dsConfig);
      LoadingService.show(' ');
      clonedDs.get().then((results: IDictionary) => {
        if (results.length > 0) {
          const productExist = results.find((el: IDictionary) => el.CDPRODUTO === valueForm.CDPRODUTO.CDPRODUTO);
          if (productExist === undefined) {
            this.callSaveProdutoSolicitacao(form, dataset);
          } else this.utilities.showAlert('alertError', 'ROW_EXISTS', 'warning');
        } else this.callSaveProdutoSolicitacao(form, dataset);
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }

  public callSaveProdutoSolicitacao(form: Form, dataset: IDictionary) {
    // this.criarLogMovopsol(dataset, 'I', '0', valueForm.QTSOLIPROD);
    LoadingService.show(' ');
    HttpCRUDTranslator.post('/produtosSolicitacao/save', { requestType: 'DataSet', dataset })
      .then((response: IDictionary) => {
      // Http.post('/NovaSolicitacao', { requestType: 'DataSet', dataset }).then((response: IDictionary) => {
      //   let solicitacao = response.data;
      //   if (!solicitacao) solicitacao = [];
      //   this.novaSolicitacao(valueForm, solicitacao);
        if (!response.data.error) {
          this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
          form.clearForm();
          const component = Metadata.getInstance<Switch>('PRODUTOEXTRA');
          component.value = false;
          this.disableJustExtra({ component });
        } else {
          const mensagem = response.data.error;
          this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
        }
      }).catch((error) => {
        this.utilities.showAlert('alert', error, 'error');
      }).finally(() => {
        LoadingService.hide();
      });
  }

  public clickDeleteSCompras({ component, event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    DialogService.show({
      name: 'deleteSCompras',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::CONFIRM_DELETE_REGISTER'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: { click: () => this.deleteSCompras({ component }) },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public deleteSCompras({ component }: IEventParam<Button>) {
    const key = component.name.split('-')[1].split('_')[0];
    const dsProdutos = DatasourceFactory.factory({
      uniqueKey: 'KEY',
      type: 'rest',
      route: '/produtosSolicitacao',
      loadAll: true,
    });
    LoadingService.show(' ');
    dsProdutos.get().then((result: IDictionary) => {
      const dataProducts = result;
      const element = dataProducts?.find((el: IDictionary) => el.KEY === key);
      const today = new Date().toJSON().slice(0, 10);
      const dataset = [
        {
          CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
          CDPRODUTO: element?.CDPRODUTO,
          NMPRODUTO: element?.NMPRODUTO,
          DTPROGENTR: element?.DTPROGENTR,
          CDFILSOLI: element?.CDFILSOLI,
          CDFILLAN: element?.CDFILLAN,
          NRSOLICMP: element?.NRSOLICMP,
          IDSOLIAPROV: element?.IDSOLIAPROV,
          KEY: key,
          IDSOLEXTRA: element?.IDSOLEXTRA,
          LOG: {
            CDFILSOLMOV: localStorage.getItem('CDFILIALLOGADA') || '',
            CDFILLANMOV: localStorage.getItem('CDFILLANCA') || '',
            NRSOLIMOV: element?.NRSOLICMP,
            CDPRODUTO: element?.CDPRODUTO,
            DTPROGENTR: element?.DTPROGENTR,
            DTOPERSOL: moment(today, 'YYYY-MM-DD').format('DD/MM/YYYY'),
            DSJUSTOPER: element?.DSJUSTOPER || null,
            DSJUSTSOLEXT: element?.DSJUSTSOLEXT || null,
            QTANTOPER: element?.QTSOLIPROD,
            QTDEPOPER: element?.QTAJUSTADA,
            IDOPERAMOVS: 'E',
            IDITMSOLEXTRA: element?.IDITMSOLEXTRA,
            changed: true,
          },
        },
      ];
      LoadingService.show(' ');
      HttpCRUDTranslator.post('/produtosSolicitacao/delete', { requestType: 'DataSet', dataset }, {}).then(() => {
        // this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green', false);
        const sCompras = Metadata.getInstance<IterableComponentRender>('iterableCardsSCompras');
        sCompras.reload();
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `${error}`, 'error');
      }).then(() => {
        DialogService.hide();
        LoadingService.hide();
      });
    });
  }

  public openFilter() {
    if (!localStorage.getItem('DATA_INICIAL') && !localStorage.getItem('DATA_FINAL')) {
      const btn = Metadata.getInstance<Button>('filterSCButton');
      btn.click();
    } else if (localStorage.getItem('CDFILIALLOGADA') !== 'null') {
      const dtIni = localStorage.getItem('DATA_INICIAL') || '';
      const dtFin = localStorage.getItem('DATA_FINAL') || '';
      this.loadSC(dtIni, dtFin, '');
    }
  }

  public clickBackSCs() {
    this.showSCDetail = false;
    const scs = Metadata.getInstance<IterableComponentRender>('iterableCardsSCompras');
    scs.reload();
  }

  public clickFilterScompras() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    const hasImportedData = localStorage.getItem('hasImportedData');
    if (filial.value && hasImportedData === 'true') {
      this.utilities.fillDatesFilter(bottomSheet);
      bottomSheet.isVisible = true;
      bottomSheet.isOpen = true;
    }
  }

  public clickCloseFilterBtn() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    bottomSheet.isVisible = false;
    bottomSheet.isOpen = false;
  }

  public setPeriodoSC({ component }: IEventParam<Button>) {
    const form = component.parent as Form;
    if (form.validate()) {
      if (form.value.PERIODO.length === 2) {
        let format = 'YYYY-MM-DD';
        if (form.value.PERIODO[1].charAt(2) === '/') format = 'DD/MM/YYYY';
        const dtini = moment(form.value.PERIODO[0], format);
        const dtfin = moment(form.value.PERIODO[1], format);
        if (dtini.isValid() && dtfin.isValid()) {
          if (dtini >= moment(localStorage.getItem('DATA_INICIAL'), 'DD/MM/YYYY')
          // && dtfin <= moment(localStorage.getItem('DATA_FINAL'), 'DD/MM/YYYY')
          ) {
            this.loadSC(dtini.format('DD/MM/YYYY'), dtfin.format('DD/MM/YYYY'), form.value.search);
            const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
            bottomSheet.isVisible = false;
            bottomSheet.isOpen = false;
          } else this.utilities.showAlert('errorPeriodFilter', 'DATE_NOT_IN_PERIOD', 'error');
        } else this.utilities.showAlert('errorDateFilter', 'INVALID_DATE', 'error');
      } else this.utilities.showAlert('errorFilter', 'FILTER_2_DATES', 'error');
    }
  }

  public loadSC(inicio: string, fim: string, search: string) {
    this.periodo.DTINICIAL = inicio;
    this.periodo.DTFINAL = fim;
    const sc = Metadata.getInstance<IterableComponentRender>('iterableCardsSCompras');
    sc.datasource.search = search;
    // sc.datasource.order = [];
    sc.reload();
  }

  public checkAccess() {
    if (localStorage.getItem('hasImportedData') === 'true') {
      const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
      if (!filial.value) {
        this.appController.unidadeLogadaIsValid();
      }
    } else {
      DialogService.show({
        name: 'needImport',
        type: 'error',
        title: I18n.translate('scm::ERROR'),
        text: I18n.translate('scm::NEED_IMPORT'),
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: I18n.translate('scm::OK'),
            events: {
              click: () => {
                DialogService.hide();
                Router.push('/importacao');
              },
            },
          },
        ],
      });
    }
  }

  public setFilterProdutos({ component }: IEventParam<Select>) {
    if (localStorage.getItem('CDFILIALLOGADA') !== 'null') {
      const ds = DatasourceFactory.factory({
        uniqueKey: 'CDFILIAL',
        type: 'rest',
        route: '/parametros',
        filter: {
          CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        },
      });
      LoadingService.show(' ');
      ds.get().then((result: IDictionary) => {
        if (result.length > 0) {
          if (!this.filiParams) this.filiParams = result.slice(0, 1).shift();
          component.datasource.filter = {
            CDFILIAL: [localStorage.getItem('CDFILIALLOGADA')] || [],
            CDGRUPPROD: localStorage.getItem('CDGRUPPROD') || 'T',
            TIPOPRODUTO: result[0]?.IDUTPRODPREF === 'N' ? 'E' : '',
          };
          component.datasource.get();
        }
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }

  public async controlDisableDeleteProducts({ component }: IEventParam<Button>) {
    if (localStorage.getItem('CDFILIALLOGADA') !== 'null' && !this.filiParams) {
      const ds = DatasourceFactory.factory({
        uniqueKey: 'CDFILIAL',
        type: 'rest',
        route: '/parametros',
        filter: {
          CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        },
      });
      LoadingService.show(' ');
      await ds.get().then((result: IDictionary) => {
        if (result.length > 0) {
          this.filiParams = result.slice(0, 1).shift();
          const dtprogentr = component.name.split('-')[1].split('_')[1];
          const idsoliaprov = component.name.split('-')[1].split('_')[2];
          if (moment(dtprogentr, 'DD/MM/YYYY') <= moment() || (idsoliaprov === 'S'
            && result[0].ALTERAR_PRODUTO_APROVADO === 'N')) {
            component.disabled = true;
          } else if (idsoliaprov === 'S') {
            component.disabled = true;
          } else component.disabled = false;
        }
      }).finally(() => {
        LoadingService.hide();
      });
    } else if (localStorage.getItem('CDFILIALLOGADA') !== 'null') {
      const dtprogentr = component.name.split('-')[1].split('_')[1];
      const idsoliaprov = component.name.split('-')[1].split('_')[2];
      if (moment(dtprogentr, 'DD/MM/YYYY') <= moment() || (idsoliaprov === 'S'
        && this.filiParams?.ALTERAR_PRODUTO_APROVADO === 'N')) {
        component.disabled = true;
      } else if (idsoliaprov === 'S') {
        component.disabled = true;
      } else component.disabled = false;
    }
  }

  public changeQuantAjustada({ component }: IEventParam<Increment>) {
    const size = component.parent?.parent?.childrenInstances?.length || 0;
    const btn = component.parent?.parent?.childrenInstances[size - 1] as Button;
    btn.disabled = false;
  }

  public changeJustQuant({ component }: IEventParam<Select>) {
    const size = component.parent?.parent?.childrenInstances?.length || 0;
    const btn = component.parent?.parent?.childrenInstances[size - 1] as Button;
    btn.disabled = false;
  }

  public async salvarEditProdSC({ component }: IEventParam<Button>) {
    const form = component.parent?.getChildInstance<Form>(component.parent?.children[0].name);
    if (form?.validate()) {
      const key = component.name.split('_')[1];
      const ds = DatasourceFactory.factory({
        uniqueKey: 'CDFILIAL',
        type: 'rest',
        route: '/parametros',
        filter: {
          CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        },
      });
      const dsProdutos = DatasourceFactory.factory({
        uniqueKey: 'KEY',
        type: 'rest',
        route: '/produtosSolicitacao',
        filter: {
          KEY: key,
        },
        loadAll: true,
      });
      LoadingService.show(' ');
      Promise.all([dsProdutos.get(), ds.get()]).then((resultP) => {
      // Promise.all([getStoredData('produtosSolicitacao'), ds.get()]).then((resultP) => {
        const dataProducts = resultP[0];
        const element = dataProducts?.find((el: IDictionary) => el.KEY === key || el.id === key);
        const dataParams = resultP[1];
        const param = dataParams?.slice(0, 1).shift();
        if (form?.value.QTAJUSTADA > element?.QTSOLIPROD && param?.IDBLOQALTSOLAVR === 'S') {
          this.utilities.showAlert('errorQuantAjustada', 'ERROR_QUANT_AJUSTADA', 'error');
          LoadingService.hide();
        } else {
          const today = new Date().toJSON().slice(0, 10);
          const dataset = [
            {
              CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
              // ...form?.value,
              QTAJUSTADA: form?.value.QTAJUSTADA || null,
              DSJUSTOPER: form?.value.DSJUSTOPER?.DSJUSTOPER || null,
              KEY: key,
              CDFILSOLI: element?.CDFILSOLI,
              CDFILLAN: element?.CDFILLAN,
              NRSOLICMP: element?.NRSOLICMP,
              CDPRODUTO: element?.CDPRODUTO,
              NMPRODUTO: element?.NMPRODUTO,
              DTPROGENTR: element?.DTPROGENTR,
              IDSOLIAPROV: element?.IDSOLIAPROV,
              IDSOLEXTRA: element?.IDSOLEXTRA,
              LOG: {
                CDFILSOLMOV: localStorage.getItem('CDFILIALLOGADA') || '',
                CDFILLANMOV: localStorage.getItem('CDFILLANCA') || '',
                NRSOLIMOV: element?.NRSOLICMP,
                CDPRODUTO: element?.CDPRODUTO,
                DTPROGENTR: element?.DTPROGENTR,
                DTOPERSOL: moment(today, 'YYYY-MM-DD').format('DD/MM/YYYY'),
                DSJUSTOPER: form?.value.DSJUSTOPER?.DSJUSTOPER || null,
                DSJUSTSOLEXT: element?.DSJUSTSOLEXT || null,
                QTANTOPER: element?.QTSOLIPROD,
                QTDEPOPER: form?.value.QTAJUSTADA,
                IDOPERAMOVS: 'A',
                IDITMSOLEXTRA: element?.IDITMSOLEXTRA,
                changed: true,
              },
            },
          ];
          LoadingService.show(' ');
          HttpCRUDTranslator.post('/produtosSolicitacao/update', { requestType: 'DataSet', dataset })
            .then((response: IDictionary) => {
              if (!response.data.error) {
                this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
                component.disabled = true;
                const tabsSC = Metadata.getInstance<Tabs>('tabsSC');
                tabsSC.nextTab();
              } else {
                const mensagem = response.data.error;
                this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
              }
            }).catch((error: string) => {
              this.utilities.showAlert('alert', error, 'error');
            }).finally(() => {
              LoadingService.hide();
            });
        }
      });
    }
  }

  public setSolicitacaoCodigo({ component }: IEventParam<Text>) {
    const value = component.name.split('-').reverse()[0] || '';
    component.text = `${I18n.translate('Solicitação')}: ${value}`;
  }
}
